--
-- HaeckslerCutter
-- This is the HaeckslerCutter
--
-- M@D Author  Headshot XXL
-- M@D date 02.09.2009
--
-- > Copyright (C) Headshot XXL - www.planet-ls.de < --

HaeckslerCutter = {};

function HaeckslerCutter.prerequisitesPresent(specializations)
    return true;
end;

function HaeckslerCutter:load(xmlFile)

    self.setReelSpeed = SpecializationUtil.callSpecializationsFunction("setReelSpeed");
    self.onStartReel = SpecializationUtil.callSpecializationsFunction("onStartReel");
    self.onStopReel = SpecializationUtil.callSpecializationsFunction("onStopReel");
    self.isReelStarted = HaeckslerCutter.isReelStarted;
	

	self.heightReferenceNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.heightReferenceNode#index"));
	
	
		
	self.cutterParticleSystems = {};
	local numCuttingAreas = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.cuttingAreas#count"), 0);
    for i=1, numCuttingAreas do
        local key = string.format("vehicle.cutterParticleSystem.cutterParticleSystem(%d)", i-1);
		local currentPS = {};
		local particleNode = Utils.loadParticleSystem(xmlFile, currentPS, key, self.components, false, nil, self.baseDirectory);
		table.insert(self.cutterParticleSystems, currentPS);
    end;
	

    self.preferedCombineSize = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.preferedCombineSize"), 1);

    self.fruitTypes = {};
    self.fruitTypes[FruitUtil.FRUITTYPE_UNKNOWN] = true;
    local fruitTypes = getXMLString(xmlFile, "vehicle.fruitTypes#fruitTypes");
    if fruitTypes ~= nil then
        local types = Utils.splitString(" ", fruitTypes);
        for k,v in pairs(types) do
            local desc = FruitUtil.fruitTypes[v];
            if desc ~= nil then
                self.fruitTypes[desc.index] = true;
            end;
        end;
    end;

    self.currentFruitType = FruitUtil.FRUITTYPE_UNKNOWN;
    self.reelStarted = false;
    self.lastArea = 0;

end;

function HaeckslerCutter:delete()

	for i=1, table.getn(self.cutterParticleSystems) do
		Utils.deleteParticleSystem(self.cutterParticleSystems[i]);
	end;
end;

function HaeckslerCutter:mouseEvent(posX, posY, isDown, isUp, button)
end;

function HaeckslerCutter:keyEvent(unicode, sym, modifier, isDown)
end;

function HaeckslerCutter:update(dt)
    self.lastArea = 0;
    if self.reelStarted and self.movingDirection < 0 then	
		for fruitType,v in pairs(self.fruitTypes) do
			for k,area in pairs(self.cuttingAreas) do
				local x,y,z = getWorldTranslation(area.start);
				local x1,y1,z1 = getWorldTranslation(area.width);
				local x2,y2,z2 = getWorldTranslation(area.height);
				local area = Utils.getFruitArea(fruitType, x, z, x1, z1, x2, z2);
				if area > 0 then
					self.currentFruitType = fruitType;
				end;
			end;
		end;
		if self.currentFruitType ~= FruitUtil.FRUITTYPE_UNKNOWN then
			local realArea = 0;
			for k,areas in pairs(self.cuttingAreas) do
				local x,y,z = getWorldTranslation(areas.start);
				local x1,y1,z1 = getWorldTranslation(areas.width);
				local x2,y2,z2 = getWorldTranslation(areas.height);
				Utils.updateFruitCutShortArea(self.currentFruitType, x, z, x1, z1, x2, z2, 1);
				local area = Utils.cutFruitArea(self.currentFruitType, x, z, x1, z1, x2, z2);
				if area > 0 then
					local spray = Utils.getDensity(g_currentMission.terrainDetailId, g_currentMission.sprayChannel, x, z, x1, z1, x2, z2);
					local multi = 1;
					if spray > 0 then
						multi = 2;
					end;
					self.lastArea = self.lastArea + area*multi;
					realArea = realArea + area/g_currentMission.maxFruitValue;
					
					Utils.setEmittingState(self.cutterParticleSystems[k], area > 0.0);
				else
					Utils.setEmittingState(self.cutterParticleSystems[k], false);
				end;
			end;
			local pixelToQm = 2048 / 4096 * 2048 / 4096; -- 4096px are mapped to 2048m
			local qm = realArea*pixelToQm;
			local ha = qm/10000;

			g_currentMission.missionStats.hectaresThreshedTotal = g_currentMission.missionStats.hectaresThreshedTotal + ha;
			g_currentMission.missionStats.hectaresThreshedSession = g_currentMission.missionStats.hectaresThreshedSession + ha;

			g_currentMission.missionStats.threshingDurationTotal = g_currentMission.missionStats.threshingDurationTotal + dt/(1000*60);
			g_currentMission.missionStats.threshingDurationSession= g_currentMission.missionStats.threshingDurationSession + dt/(1000*60);
		end;	
	else
		for i=1, table.getn(self.cutterParticleSystems) do
			Utils.setEmittingState(self.cutterParticleSystems[i], false);
		end;
    end;
	

	
end;

function HaeckslerCutter:draw()
end;

function HaeckslerCutter:onDetach()
    if self.deactivateOnDetach then
        HaeckslerCutter.onDeactivate(self);
    end;
end;

function HaeckslerCutter:onLeave()
    if self.deactivateOnLeave then
        HaeckslerCutter.onDeactivate(self);
    end;
end;

function HaeckslerCutter:onDeactivate()
    self:onStopReel();
    Utils.setEmittingState(self.threshingParticleSystems, false);
end;

function HaeckslerCutter:setReelSpeed(speed)
    self.reelSpeed = speed;
end;

function HaeckslerCutter:onStartReel()
    self.reelStarted = true;
end;

function HaeckslerCutter:onStopReel()
    self.reelStarted = false;
    Utils.setEmittingState(self.threshingParticleSystems, false);
end;

function HaeckslerCutter:isReelStarted()
    return self.reelStarted;
end;
